;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************

	TITLE           "GNU-link : CD-Multichanger Emulation"
	SUBTITLE        "Released under the GPL license, (c) S Wood, Oct 2001"

;******************************************************************************
; GNUnilink Configuration.
; Uncomment the lines relivent to your board... 
;
; this should be only configuration required.

; Type of PIC (or closest)
;#define		PIC_12C509
;#define		PIC_16F84
;#define		PIC_16F627
#define		PIC_16F628

; Speed (or closest)
;#define		SPEED_20MHZ
;#define		SPEED_16MHZ
#define		SPEED_10MHZ
;#define		SPEED_8MHZ
;#define		SPEED_4MHZ

; Emulate a MiniDisk Changer, rather than a CD Changer
#define		MINIDISK

; Serial Output - report the button presses on the head unit to the PC via
; serial port
#define		SERIAL_OUTPUT

; Serial Input - allow PC to change Disk/Track Names/Numbers and report Track 
; Counter via serial port
#define		SERIAL_INPUT

; Debug - should not be set for 12c509 as the're aren't enough pins on the 
; device!! Should not be used with SERIAL_OUTPUT as there is pin contention.
#define		DEBUG


;******************************************************************************
; Real settings for the PIC - if you need to add a new one do it here, 
; otherwise select one from above

		Radix   DEC
		EXPAND

	IFDEF	PIC_12C509
		Processor       12c509
		include         "p12c509.inc"

	__CONFIG _CP_OFF & _WDT_OFF & _XT_OSC

#define		OUT_PORT	GPIO
#define		PIC_12
	ENDIF

	IFDEF	PIC_16F84
		Processor       16f84
		include         "p16f84.inc"

	__CONFIG _CP_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC

#define		PIC_16
	ENDIF

	IFDEF	PIC_16F627
		Processor       16f627
		include         "p16f627.inc"

	__CONFIG _CP_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC

#define		PIC_16F62x
#define		PIC_16

; Nast Hack to only use timing for the slave break
#define		NASTY_HACK
	ENDIF

	IFDEF	PIC_16F628
		Processor       16f628
		include         "p16f628.inc"

	__CONFIG _CP_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC & _LVP_OFF

#define		PIC_16F62x
#define		PIC_16
	ENDIF

	IFDEF	PIC_16
#define		OUT_PORT	PORTB
#define		OUT_ENA		TRISB
	ENDIF

;******************************************************************************
; A few standard definitions

_ResetVector	set	0x00
_IntVector      set     0x04

TRUE	equ	1
FALSE	equ	0

;******************************************************************************
; Timing Calibration Values - These are non critical in that they don't have 
; to be absolutely correct, but it's nicer to be almost right if you can....

; 100us delay loop is 8 + (n * 7) cycles.

	IFDEF	SPEED_4MHZ
#define		delay100us	13
#define		baud_val	12	; 19231 baud
	ENDIF

	IFDEF	SPEED_8MHZ
#define		delay100us	27
#define		baud_val	25	; 19231 baud
	ENDIF

	IFDEF	SPEED_10MHZ
#define		delay100us	34
#define		baud_val	7	; 78125 baud
	ENDIF

	IFDEF	SPEED_16MHZ
#define		delay100us	56
#define		baud_val	51	; 19231 baud
	ENDIF

	IFDEF	SPEED_20MHZ
#define		delay100us	70
#define		baud_val	64	; 19231 baud
	ENDIF

;******************************************************************************
	IFDEF	SERIAL_INPUT
#define	SERIAL
	ENDIF

	IFDEF	SERIAL_OUTPUT
	IFNDEF	SERIAL
#define	SERIAL
	ENDIF
	ENDIF

	IFDEF	SERIAL
#define	USE_INT				; enable interrupt service routine
	ENDIF

;******************************************************************************
;                               Setup Register Locations

DataVal		set		0x20	; used by ReadByte/SendByte
Count		set		0x21
Count2		set		0x22
Checksum	set		0x23

Rad		set		0x24	; filled by receive function
Tad		set		0x25
Cmd1		set		0x26
Cmd2		set		0x27
Parity1		set		0x28
Data11		set		0x29
Data12		set		0x2a
Data13		set		0x2b
Data14		set		0x2c
Data21		set		0x2d
Data22		set		0x2e
Data23		set		0x2f
Data24		set		0x30
Data25		set		0x31
Parity2		set		0x32
EndByte		set		0x33

MyId		set		0x34
MyBit		set		0x35
DisplayW	set		0x36	; Pointer to write into display buffer
DisplayR	set		0x37	; Pointer to read from the display buffer
State		set		0x38	; 0x80: Idle;  0x00: Playing
SomeFlags	set		0x39	; bit0: SBRQ (SlaveBreak request)
; SomeFlags bits:
SBRQ		set		0

;*** data values to send to headunit ***
;those must be in located in one memory block (Bank0); their order is presented directly to the host  (see interrupt.asm)
RX_FirstLoc	set		DiskCount	; used by IntRX_Receive to access "write" memory block
RX_LastLoc	set		TXPlayMode ;UserBuf3
RX_AddrOfs	set		0	; lowest (virtual) address that the host can write to

DiskCount	set		0x3A	; (was soll das eigentlich?  030422)
DiskNum		set		0x3B	; 0-9 (0 is displayed as 10)
Time0Total	set		0x3C	; Disk length (seconds, BCD)
Time1Total	set		0x3D	;  "          (minutes, binary)
TrackCount	set		0x3E	; Track count (binary)
TrackNum	set		0x3F	; Track Number
Time0		set		0x40	; Track Timer 0:00-99:99
Time1		set		0x41	; 

UserBuf0	set		0x42	; memory can be read/written by host to save its playing position or similar
UserBuf1	set		0x43	; evtl. in EEPROM speichern; eigentlich aber nicht ntig da PIC immer Strom hat
UserBuf2	set		0x44	; 
UserBuf3	set		0x45	; 

;those must be in located in one memory block (Bank0); their order is presented directly to the host  (see interrupt.asm)
TX_FirstLoc	set		DiskCount	; used by IntRX_Transmit to access "read" memory block
TX_LastLoc	set		TXPlayMode
TX_AddrOfs	set		0	; lowest (virtual) address that the host can read from

TXDiskNum	set		0x46	; Disk-Number (direct disk select); zero if no disk change after last host query
TXTrackNum	set		0x47	; direct track select; zero if...
TXTrackRel	set		0x48	; signed int; relative track select (Auswahlrad am Radio)
TXPlayMode	set		0x49	; REPEAT, RANDOM, STOP[Flag set on device deselect; Host should shut down
TXPM_RPT	set		0	; and send PIC command to turn power off, maybe after some minutes delay]
TXPM_RP		set		1	; Shuffle mode
TXPM_Intro	set		2	; Intro scan active
TXPM_FF		set		3	; Fast forward pressed
TXPM_REV	set		4	; Reverse pressed
TXPM_Stop	set		7	; not yet used
TXTrackRel_B	set		0x4A	; background field (written by parse.asm)

	IFDEF	SERIAL_INPUT
RXChar		set		0x50
RXPointer	set		0x51
	ENDIF

;*** 30 bytes ring buffer ***
Display0	set		0x52
DisplayEnd	set		0x6f

	IFDEF	USE_INT
status_store	set		0x70	; stores STATUS reg during interrupt
w_store		set		0x71	; stores w reg during interrupt
	ENDIF

SerByte		set		0x72	; used by SendSerial function
Count3		set		0x73	; SendSerial cannot use Count2 because sometimes C2 is used to save the Checksum over a SendByte call

DiskName0	set		0xa0	; bank 1!
DiskName1	set		0xa1
DiskName2	set		0xa2
DiskName3	set		0xa3
DiskName4	set		0xa4
DiskName5	set		0xa5
DiskName6	set		0xa6
DiskName7	set		0xa7

DiskName8	set		0xa8
DiskName9	set		0xa9
DiskNameA	set		0xaa
DiskNameB	set		0xab
DiskNameC	set		0xac
DiskNameD	set		0xad
DiskNameE	set		0xae
DiskNameF	set		0xaf

TrackName0	set		0xb0
TrackName1	set		0xb1
TrackName2	set		0xb2
TrackName3	set		0xb3
TrackName4	set		0xb4
TrackName5	set		0xb5
TrackName6	set		0xb6
TrackName7	set		0xb7

TrackName8	set		0xb8
TrackName9	set		0xb9
TrackNameA	set		0xba
TrackNameB	set		0xbb
TrackNameC	set		0xbc
TrackNameD	set		0xbd
TrackNameE	set		0xbe
TrackNameF	set		0xbf

;******************************************************************************
; Port definitions, which signal is where on the OUT_PORT
#define		DataBit		5
#define 	ClockBit	3
#define 	BusOnBit	6

; Define which group you are
	IFDEF	MINIDISK
#define		MyGroup		0xD0		;I'm a MiniDisk Mulitchanger
	ELSE
#define		MyGroup		0x30		;I'm a CD Mulitchanger
	ENDIF

;******************************************************************************
; Default definations for names etc.

TrackName0_def	set		'<'		;Track Name
TrackName1_def	set		'A'
TrackName2_def	set		'U'
TrackName3_def	set		'X'
TrackName4_def	set		'-'
TrackName5_def	set		'I'
TrackName6_def	set		'N'
TrackName7_def	set		'>'
TrackName8_def	set		' '
TrackName9_def	set		' '
TrackNameA_def	set		' '
TrackNameB_def	set		' '
TrackNameC_def	set		' '
TrackNameD_def	set		' '
TrackNameE_def	set		' '
TrackNameF_def	set		' '

DiskName0_def	set		'A'		;Disk Name
DiskName1_def	set		'd'
DiskName2_def	set		'a'
DiskName3_def	set		'm'
DiskName4_def	set		' '
DiskName5_def	set		'3'
DiskName6_def	set		'0'
DiskName7_def	set		'1'
DiskName8_def	set		' '
DiskName9_def	set		' '
DiskNameA_def	set		' '
DiskNameB_def	set		' '
DiskNameC_def	set		' '
DiskNameD_def	set		' '
DiskNameE_def	set		' '
DiskNameF_def	set		' '

Time1Total_def	set		75		; Disk length (minutes, binary)
Time0Total_def	set		0x59		;  "          (seconds, BCD)
DiskCount_def	set		6 		; 
TrackCount_def	set		1		; Track count (binary)

Time1_def	set		00		; Track Timer 0:00-99:99
Time0_def	set		0x00		; 
DiskNum_def	set		1 		; 0-9 (0 is displayed as 10)

	IFDEF	MINIDISK
TrackNum_def	set		0x01		; Track Number 0-255
	ELSE
TrackNum_def	set		0xf1		; Track Number 0-99
	ENDIF					; ('f' displays ' ')

;******************************************************************************
; Debug option - Outputs a value on PORTB to indicate the state of the system
; Not possible with 12C509 as there aren't enough pins!!
; Pin connetion with SERIAL_OUTPUT

	IFDEF	DEBUG
#define		StateBreak	0x00
#define		StateReceive	0x10
#define		StateInterpret	0x20
#define 	StateAnyone	0x30
#define		StateHello	0x40
#define		StatePing	0x50
#define		StateAppoint	0x60
#define		StateNak	0x70
#define		StateWillis	0x80
#define		StateGetPos	0x90

#define		StateResume	0xa0
#define		StateHold	0xb0

#define		StateUnkown	0x00
	ENDIF

;******************************************************************************
; Start of code space

	ORG     _ResetVector
	
	goto	SystemSetup

;******************************************************************************

	include "interrupt.asm"
	include "functions.asm"
	include "setup.asm"
	include "receive.asm"
	include "parse.asm"
	include	"respond.asm"
	include "display.asm"

;******************************************************************************

	END
