;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
; Sequenced reponses to the Slave Poll, called so that the screen is updated.

Respond_SlavePoll:
	movf	DisplayR, 0			; Nothing to display (R=W)
	xorwf	DisplayW, 0
	btfsc	STATUS, Z
	goto	Respond_Ping			; send current state

	IFDEF	DEBUG
	movlw	0xAE				; 0xAE: start new answer packet
	call	SendSerial
	ENDIF

	movf	MyId, 0				; Check it's the right ID
	subwf	Rad, 0
	btfss	STATUS, Z
	goto	ReceiveBreak			; Done, wait for next command

	clrf	Checksum 			; Clear checksum

	call	DB_ReadPrep			; fetch byte from display queue

	movf	INDF, 0				; send TIME info
	andlw	0xF0
	xorlw	0x10
	btfsc	STATUS, Z
	goto	Respond_SlavePoll_TIME

	movf	INDF, 0				; send SEEK info
	andlw	0xF0
	xorlw	0x20
	btfsc	STATUS, Z
	goto	Respond_SlavePoll_SEEKINFO

	movf	INDF, 0				; send DISCTIME info
	andlw	0xF0
	xorlw	0x30
	btfsc	STATUS, Z
	goto	Respond_SlavePoll_DISCTIME

	movf	INDF, 0				; send DISCINFO
	andlw	0xF0
	xorlw	0x40
	btfsc	STATUS, Z
	goto	Respond_SlavePoll_DISCINFO


	movf	INDF, 0				; send track name
	andlw	0xE0				; 0x60/0x70
	xorlw	0x60
	btfsc	STATUS, Z
	goto	Respond_SlavePoll_TrackName

	movf	INDF, 0				; send disk name
	andlw	0xE0				; 0x80/0x90
	xorlw	0x80
	btfsc	STATUS, Z
	goto	Respond_SlavePoll_DiskName

	goto	Respond_Ping			; unhandled Display value

;--------

Respond_SlavePoll_TIME:				; display code:  0x10
	movlw	0x70				; 'TIME' (displayflags=3; trackflags=0x50):
				; 0x70, MyID, 0x90, [v_cdc_trackflags], CS, [v_cdc_track], [v_cdc_minutes], [v_cdc_seconds],
				; ([v_cdc_disc] shl 4) or [v_cdc_displayflags], CS
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0x90
	call	SendByte

	movlw	0x50; 0x10; 0				; Track flags(?)
	call	SendByte

	movf	Checksum, 0
	movwf	Count2

	movf	Checksum, 0
	call	SendByte

	movf	Count2, 0
	movwf	Checksum
	
	movf	TrackNum, 0			; Track Number
	call	SendByte

	movf	Time1, 0			; Minutes
	call	SendByte

	movf	Time0, 0			; Seconds
	call	SendByte

	swapf	DiskNum, 0			; Store disk number into hi nibble
	movwf	Count2
	movf	INDF, 0				; Load display flags from command byte (lo nibble)
	andlw	0x0F
	iorwf	Count2, 0
	call	SendByte

	goto	Slave_Respond_End

;--------

Respond_SlavePoll_SEEKINFO:			; display code:  0x20
	movlw	0x70  ;0x77				; 'SEEKINFO' (temp2=0; displayflags=3; trackflags=0x50):
				; 0x77, MyID, 0xc0, [temp2], CS, 0, 0, 0, 0, [v_cdc_trackflags], [v_cdc_track],
				; [v_cdc_minutes], [v_cdc_seconds], ([v_cdc_disc] shl 4) or [v_cdc_displayflags], CS
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0xc0
	call	SendByte

	movlw	0x00
	call	SendByte

	movf	Checksum, 0
	movwf	Count2

	movf	Checksum, 0
	call	SendByte

	movf	Count2, 0
	movwf	Checksum
	
	movlw	0x00	;0x09			; repeat mode?  (ArneH/unilink_info.txt)
	call	SendByte

	movlw	0x00	;0x90			; shuffle mode? (ArneH)
	call	SendByte

	movlw	0x00
	call	SendByte

	movlw	0x00
	call	SendByte
	
	movlw	0x10  ;0  ;0x50				; Track flags(?)
	call	SendByte
	
	movf	TrackNum, 0			; Track Number
	call	SendByte

	movf	Time1, 0			; Minutes
	call	SendByte

	movf	Time0, 0			; Seconds
	call	SendByte

	swapf	DiskNum, 0			; Store disk number into hi nibble
	movwf	Count2
	movf	INDF, 0				; Load display flags from command byte (lo nibble)
	andlw	0x0F
	iorwf	Count2, 0
	call	SendByte

	goto	Slave_Respond_End

;--------

Respond_SlavePoll_DISCTIME:			; display code:  0x30
	movlw	0x70				; 'DISCTIME' (displayflags=3; trackflags=0x50):
				; C_DISPADDR[0x70], MyID, C_CMD_DISCTIME[0x97], [v_cdc_trackflags], CS, [v_cdc_trackcount], [v_cdc_minutesALL],
				; [v_cdc_secondsALL], ([v_cdc_disccount] shl 4) or [v_cdc_displayflags], CS
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0x97
	call	SendByte

	movlw	1;2  ;0x50			; maybe lowest track number? (should then be normally 1)
	call	SendByte

	movf	Checksum, 0
	movwf	Count2

	movf	Checksum, 0
	call	SendByte

	movf	Count2, 0
	movwf	Checksum
	
	movf	TrackCount, 0			; Total track count
;030423	addwf	DiskNum, 0		;just for testing to see track count change when selecting disks
	call	SendByte

	movf	Time1Total, 0			; Total minutes
	call	SendByte

	movf	Time0Total, 0			; Total seconds
	call	SendByte

	swapf	DiskCount, 0			; Store disk count into hi nibble
	movwf	Count2
	movf	INDF, 0				; Load display flags from command byte (lo nibble)
	andlw	0x0F
	iorwf	Count2, 0
	call	SendByte
	
	goto	Slave_Respond_End

;--------

Respond_SlavePoll_DISCINFO:			; display code:  0x40
	movf	Tad, 0				; 'DISCINFO' (discflags=0xFFFF):
				; [Tad], MyID, C_CMD_DISCINFO[0x95], [v_cdc_discflags], CS, [v_cdc_discflags+1], 0, 0x0F, 
				; [v_cdc_disccount] or 0xF0, CS
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0x95
	call	SendByte

	movlw	0xFF
	call	SendByte

	movf	Checksum, 0
	movwf	Count2

	movf	Checksum, 0
	call	SendByte

	movf	Count2, 0
	movwf	Checksum
	
	movlw	0xFF
	call	SendByte

	movlw	0x00
	call	SendByte

	movlw	0x0F
	call	SendByte

	movf	DiskCount, 0			; Total disk count or 0xF0
	iorlw	0xF0
	call	SendByte
	
	goto	Slave_Respond_End

;--------

Respond_SlavePoll_DiskName:			; display code:  0x80/0x90
	movlw	0x70				; 'DISCNAME' (temp2=00; displayflags=0x0e):
				; 0x70, MyID, 0xcd, txt0, CS, txt1, txt2, txt3, txt4, txt5, txt6, txt7, [v_cdc_disc], 
				; [displayflags] or ([temp2] shl 4), CS
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0xcd
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName0, 0
	bcf	STATUS, RP0
	call	SendByte

	movf	Checksum, 0
	movwf	Count2

	movf	Checksum, 0
	call	SendByte

	movf	Count2, 0
	movwf	Checksum
	
	bsf	STATUS, RP0
	movf	DiskName1, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName2, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName3, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName4, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName5, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName6, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	DiskName7, 0
	bcf	STATUS, RP0
	call	SendByte

	movf	DiskNum, 0			; Disk number
	call	SendByte

	movf	INDF, 0				; see Arne Hennig's-code
	andlw	0x1F
	call	SendByte

	goto	Slave_Respond_End

;--------

Respond_SlavePoll_TrackName:			; display code:  0x60/0x70
	movlw	0x70				; 'TRACKNAME' (temp2=00; displayflags=0x0e):
				; 0x70, MyID, 0xc9, txt0, CS, txt1, txt2, txt3, txt4, txt5, txt6, txt7, [v_cdc_track], 
				; [displayflags] or ([temp2] shl 4), CS
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0xc9
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName0, 0
	bcf	STATUS, RP0
	call	SendByte

	movf	Checksum, 0
	movwf	Count2

	movf	Checksum, 0
	call	SendByte

	movf	Count2, 0
	movwf	Checksum
	
	bsf	STATUS, RP0
	movf	TrackName1, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName2, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName3, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName4, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName5, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName6, 0
	bcf	STATUS, RP0
	call	SendByte

	bsf	STATUS, RP0
	movf	TrackName7, 0
	bcf	STATUS, RP0
	call	SendByte

	movf	TrackNum, 0			; Track number
	call	SendByte

	movf	INDF, 0				; see Arne Hennig's-code
	andlw	0x1F
	call	SendByte

	goto	Slave_Respond_End

;--------

Slave_Respond_End:
	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte
	
	movf	DisplayR, 0			; more data to send to display? (R!=W)
	xorwf	DisplayW, 0
	btfss	STATUS, Z
	call	IssueSlaveBreak			; cause next_display to appear quickly
	
;	movf	DisplayR, 0			; no more data to send to display? (R==W)
;	xorwf	DisplayW, 0
;	btfsc	STATUS, Z
;	bcf	SomeFlags, SBRQ			; clear flag

	goto	ReceiveBreak			; Done, wait for next command

