;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
; Interupt Vector - used to process serial data

	IFDEF	USE_INT
	ORG	_IntVector
	movwf	w_store				; store w register
	swapf	STATUS,W
	bcf	STATUS,RP0
	movwf	status_store			; store status register

	IFDEF	SERIAL_INPUT
	btfsc	PIR1, RCIF			; test if RX interrupt is set
	goto	IntRX
	ENDIF

	btfsc	PIR1, EEIF			; test if EE interrupt is set
	goto	IntEE

IntExit:
	bcf	STATUS, RP0
	swapf	status_store,W			; restore registers
	movwf	STATUS
	swapf	w_store,F
	swapf	w_store,W    
        
	RETFIE					; re enable interrupts
        
	IFDEF	SERIAL_INPUT
IntRX:						; RX interrupt handler
	bcf	PIR1, RCIF			; clear int flag

	btfsc	RCSTA, OERR			; check for overflow error
	goto	IntRXOvErr

	btfsc	RCSTA, FERR			; check for framing error (byte error)
	goto	IntRXFrameErr

	btfss	RCSTA, RX9D			; Bit9=1: Address byte
	goto	IntRXData
;IntRXAddress:
	movf	RCREG, W			; read RX'ed charater (free receive buffer)
	movwf	RXChar

	clrf	RXPointer			; will be re-set if correct WRITE location received

	btfss	RXChar, 7			; RXChar >= 0x80: read location (PIC transmits)
	goto	IntRXAdWrite
;IntRXAdRead:
	bcf	RXChar, 7			; subtract 0x80

	movlw	0x40				; address >= 0x40? --> read disk/track name
	subwf	RXChar, W
	BTFSC	STATUS, C			; [C = (RXChar >= 0x40)]
	goto	IntRXAdReadName	

	movlw	TX_AddrOfs			; subtract address offset
	subwf	RXChar, F
	btfss	STATUS, C			; invalid location addressed
	goto	IntRXEnd
	movlw	TX_FirstLoc			; add TX_FirstLoc
	addwf	RXChar, F

	movlw	TX_LastLoc + 1			; ensure address isn't too high
	subwf	RXChar, W			; [sets carry=0 if (f-w)<0]
	btfsc	STATUS, C
	goto	IntRXEnd			; invalid location addressed

	movf	RXChar, W			; set up Indirect addressing
	movwf	FSR
	bcf	STATUS, IRP

	;movf	RXChar, W			; check if TXTrackRel addressed
	xorlw	TXTrackRel
	btfss	STATUS, Z
	goto	IntRXAdRead_NotTrackRel
	movf	TXTrackRel, W			; TXTrackRel = 0?
	btfss	STATUS, Z
	goto	IntRXAdRead_NotTrackRel
	movf	TXTrackRel_B, W			; load & reset background value
	movwf	TXTrackRel
	clrf	TXTrackRel_B
IntRXAdRead_NotTrackRel:
	movf	INDF, W				; transmit desired register data
	movwf	TXREG

;	movlw	TXDiskNum			; reset locations above UserBufX on read
;	subwf	RXChar, W			; [sets carry=0 if (f-w)<0]
;	btfsc	STATUS, C
;	clrf	INDF

	goto	IntRXEnd

IntRXAdReadName:
	addlw	DiskName0			; add DiskName0 to W = RXChar-0x40
	movwf	RXChar

	movlw	TrackNameF + 1			; ensure address isn't too high
	subwf	RXChar, W			; [sets carry=0 if (f-w)<0]
	btfsc	STATUS, C
	goto	IntRXEnd			; invalid location addressed

	movf	RXChar, W			; set up Indirect addressing
	movwf	FSR

	goto	IntRXAdRead_NotTrackRel		; send data

IntRXAdWrite:
	movf	RXChar, W			; send back received address for error-checking
	movwf	TXREG

	movlw	0x40				; address >= 0x40? --> write disk/track name
	subwf	RXChar, W
	BTFSC	STATUS, C			; [C = (RXChar >= 0x40)]
	goto	IntRXAdWriteName	

;031001: TXTrackRel can now be written, so this exception isn't necessary anymore (but: Beck must send a 0 after address!)
;	movf	RXChar, W			; check if TXTrackRel addressed
;	xorlw	TXTrackRel - TX_FirstLoc + TX_AddrOfs
;	btfsc	STATUS, Z
;	clrf	TXTrackRel			; clear field once it is addressed; so no data needs to be sent after address

	movf	RXChar, W			; check for DISPLAY location
	xorlw	0x3F				; =63dec
	btfsc	STATUS, Z
	goto	IntRXAdWriteDisp

	movlw	RX_AddrOfs			; subtract address offset
	subwf	RXChar, F
	btfss	STATUS, C			; invalid location addressed
	goto	IntRXEnd
	movlw	RX_FirstLoc			; add RX_FirstLoc
	addwf	RXChar, F

	movlw	RX_LastLoc + 1			; ensure address isn't too high
	subwf	RXChar, W			; [sets carry=0 if (f-w)<0]
	btfsc	STATUS, C
	goto	IntRXEnd			; invalid location addressed

	movf	RXChar, W			; save destination pointer
	movwf	RXPointer

	goto	IntRXEnd

IntRXAdWriteName:
	addlw	DiskName0			; add DiskName0 to W = RXChar-0x40
	movwf	RXChar

	movlw	TrackNameF + 1			; ensure address isn't too high
	subwf	RXChar, W			; [sets carry=0 if (f-w)<0]
	btfsc	STATUS, C
	goto	IntRXEnd			; invalid location addressed

	movf	RXChar, W			; save destination pointer
	movwf	RXPointer

	goto	IntRXEnd

IntRXAdWriteDisp:
	movlw	Display0			; used as a flag
	movwf	RXPointer

	goto	IntRXEnd

IntRXData:
	movf	RCREG, W			; read RX'ed charater (free receive buffer)
	movwf	RXChar

	movf	RXPointer, F			; test if destination specified
	btfsc	STATUS, Z
	goto	IntRXEnd

	movf	RXChar, W			; send back received value for error-checking
	movwf	TXREG

	movf	RXPointer, W			; check for DISPLAY location
	xorlw	Display0
	btfsc	STATUS, Z
	goto	IntRXDataDisplay

	movf	RXPointer, W			; set up Indirect addressing
	movwf	FSR
	bcf	STATUS, IRP

	movf	RXChar, W			; store received byte
	movwf	INDF

;immer zurcksetzen
;	incf	RXPointer, F			; set pointer to next byte

;	movlw	RX_LastLoc + 1			; ensure address isn't too high
;	subwf	RXPointer, W			; [sets carry=0 if (f-w)<0]
;	btfsc	STATUS, C
	clrf	RXPointer			; new address is too high

	goto	IntRXEnd

IntRXDataDisplay:
	clrf	RXPointer

	movf	RXChar, W			; check which display to update
	xorlw	0x01				; =dec
	btfsc	STATUS, Z
	goto	IntRXDataDisp_DiskNum

	movf	RXChar, W
	xorlw	0x02				; =dec
	btfsc	STATUS, Z
	goto	IntRXDataDisp_TrackNum

	movf	RXChar, W
	xorlw	0x03				; =dec
	btfsc	STATUS, Z
	goto	IntRXDataDisp_Counter

	movf	RXChar, W			; disk name: update counter (03) must be sent afterwards!
	xorlw	0x04				; =dec
	btfsc	STATUS, Z
	goto	IntRXDataDisp_DiskName

	goto	IntRXEnd

IntRXDataDisp_DiskNum:
	bsf	SomeFlags, SBRQ			; request a SlaveBreak

	call	DB_WritePrep
	movlw	0x20
	movwf	INDF

	call	DB_WritePrep
	movlw	0x13
	movwf	INDF

	call	DB_WritePrep			; Send total disk time
	movlw	0x33
	movwf	INDF

IntRXDataDisp_DiskName:
	call	DB_WritePrep			; Send disk name [0-7]
	movlw	0x83
	movwf	INDF

	goto	IntRXEnd

IntRXDataDisp_TrackNum:
	bsf	SomeFlags, SBRQ			; request a SlaveBreak

	call	DB_WritePrep
	movlw	0x20
	movwf	INDF

	goto	IntRXEnd

IntRXDataDisp_Counter:
	bsf	SomeFlags, SBRQ			; request a SlaveBreak

	call	DB_WritePrep			; Send current time
	movlw	0x1e
	movwf	INDF

	goto	IntRXEnd

IntRXOvErr:
	bcf	RCSTA, CREN			; reset OERR flag
	bsf	RCSTA, CREN
;folgendes noch nicht endgltig:
	movf	RCREG, W			; discard RX'ed charater (free receive buffer)
	movf	RCREG, W			; discard RX'ed charater (free receive buffer)
	movf	RCREG, W			; discard RX'ed charater (free receive buffer)
	clrf	RXPointer			; reset address pointer
	goto	IntRXEnd

IntRXFrameErr:
	movf	RCREG, W			; discard RX'ed charater (free receive buffer)
	clrf	RXPointer			; reset address pointer
	;goto	IntRXEnd

IntRXEnd:
	goto	IntExit
	ENDIF

IntEE:						; EEPROM interrupt handler
	bcf	PIR1, EEIF			; clear int flag

	bsf	STATUS, RP0			; Bank 1
	movf	EEADR, W			; check if another byte to write
	sublw	2				; C := (2-W >= 0)
	btfss	STATUS, C
	goto	IntEEEnd			; EEADR > 2: writing finished
;IntEEWriteNext:
	incf	EEADR, F			; write next EEPROM location

	movf	EEADR, W			; set up Indirect addressing
	addlw	UserBuf0
	movwf	FSR
	bcf	STATUS, IRP

	movf	INDF, W				; load & write next UserBuf value
	call	EEWrite

	;goto	IntEEEnd

IntEEEnd:
	goto	IntExit

	ENDIF
