;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
; Various responses to the commands
; These are clocked out by the master when it is ready

Respond_Anyone:
	IFDEF	DEBUG
	movlw	0xAE				; 0xAE: start new answer packet
	call	SendSerial
	ENDIF

	movf	MyId, 1				; Test if we've got an ID
	btfss	STATUS, Z
	goto	ReceiveBreak			; we don't need another one

	clrf	Checksum 			; Clear checksum

	movf	Tad, 0				; 'I'm a CD/MD player'
	call	SendByte

	movlw	MyGroup
	call	SendByte

Respond_Anyone_Appoint:				; same answer for Anyone/Appoint
	movlw	0x8c
	call	SendByte

	movlw	0x10
	call	SendByte

	movf	Checksum, 0			; Save Current Checksum
	movwf	Count2

	movf	Checksum, 0			; Send Checksum
	call	SendByte

	movf	Count2, 0			; Restore Checksum
	movwf	Checksum
	
	movlw	0x24
	call	SendByte

	IFDEF	MINIDISK
	movlw	0xAC
	ELSE
	movlw	0xA8
	ENDIF
	call	SendByte

	movlw	0x17
	call	SendByte

	IFDEF	MINIDISK
	movlw	0xA0
	ELSE
	movlw	0x60
	ENDIF
	call	SendByte
	
Respond_End:
	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte
	
	goto	ReceiveBreak			; Done, wait for next command

;--------

Respond_Appoint:
	IFDEF	DEBUG
	movlw	0xAE				; 0xAE: start new answer packet
	call	SendSerial
	ENDIF

	movf	Rad, 0				; respond if its my group...
	andlw	0xf0
	xorlw	MyGroup
	btfss	STATUS, Z
	goto	ReceiveBreak			; Not MyGroup

	movf	Rad, 0				; Store allocated ID
	movwf	MyId

	movf	Cmd2, 0				; Store bit position 
	movwf	MyBit				; (for SlavePoll response)

	clrf	Checksum 			; Clear checksum

	movf	Tad, 0				; 'OK I'm ID=X'
	call	SendByte

	movf	MyId, 0
	call	SendByte

	goto	Respond_Anyone_Appoint

;--------

Respond_Hello:
	movf	MyId, 1				; Test for ID
	btfss	STATUS, Z
	goto	ReceiveBreak			; keep quiet if we have...

	IFDEF	DEBUG
	movlw	0xAE				; 0xAE: start new answer packet
	call	SendSerial
	ENDIF

	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'You've missed me!!'
	call	SendByte

	movlw	0x18
	call	SendByte

	movlw	0x04
	call	SendByte

	movlw	0x00
	call	SendByte

	goto	Respond_End

;--------

Respond_Ping:
	IFDEF	DEBUG
	movlw	0xAE				; 0xAE: start new answer packet
	call	SendSerial
	ENDIF

	movf	MyId, 0				; Check it's the right ID
	subwf	Rad, 0
	btfss	STATUS, Z
	goto	ReceiveBreak			; Done, wait for next command

	clrf	Checksum 			; Clear checksum

	movf	Tad, 0				; Send current state
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0x00
	call	SendByte

	movf	State, 0			; 0x80='Idle', 0x00='Playing'
	call	SendByte

	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte

	goto	ReceiveBreak			; wait for next command

;******************************************************************************
; Response to Master Poll Command

Respond_MasterPoll:
	IFDEF	DEBUG
	movlw	0xAE				; 0xAE: start new answer packet
	call	SendSerial
	ENDIF

	movf	DisplayR, 0			; display buffer empty?
	xorwf	DisplayW, 0
	btfsc	STATUS, Z
	goto	ReceiveBreak			; wait for next command
	
	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'Something has changed!'
	call	SendByte

	movlw	0x18				; is to a broadcast address
	call	SendByte

	movlw	0x82
	call	SendByte

	movlw	0x00				; MyBit is 0x0? or 0x1?
	call	Bit_Frig
	call	SendByte

	movf	Checksum, 0			; Save Current Checksum
	movwf	Count2

	movf	Checksum, 0			; Send Checksum
	call	SendByte

	movf	Count2, 0			; Restore Checksum
	movwf	Checksum
	
	movlw	0x20				; MyBit is 0x2? or 0x3?
	call	Bit_Frig
	call	SendByte

	movlw	0x40				; MyBit is 0x4? or 0x5?
	call	Bit_Frig
	call	SendByte

	movlw	0x60				; MyBit is 0x6? or 0x7?
	call	Bit_Frig
	call	SendByte

	movlw	0x80				; MyBit is 0x8? or 0x9?
	call	Bit_Frig
	call	SendByte
	
	goto	Respond_End			; to do checksum and error byte

;******************************************************************************

