;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************

; Initialisation of all the variables, called on power up on on a bus reset.

SystemSetup:
	bcf	STATUS, RP0
	bcf	STATUS, RP1

	movlw	DiskCount_def
	movwf	DiskCount
	movlw	DiskNum_def
	movwf	DiskNum
	movlw	Time0Total_def
	movwf	Time0Total
	movlw	Time1Total_def
	movwf	Time1Total

	movlw	TrackCount_def
	movwf	TrackCount
	movlw	TrackNum_def
	movwf	TrackNum
	movlw	Time0_def
	movwf	Time0
	movlw	Time1_def
	movwf	Time1

	movlw	UserBuf0			; read UserBuf from EEPROM
	movwf	FSR
	bcf	STATUS, IRP
	
	bsf	STATUS, RP0			; Bank 1
	clrf	EEADR

SystemSetup_EELoop:
	bsf	EECON1, RD			; read from EEPROM
	movf	EEDATA, W
	movwf	INDF

	incf	EEADR, F			; calculate next addresses
	incf	FSR

	btfss	EEADR, 2			; end loop if EEADR = 4
	goto	SystemSetup_EELoop

	bcf	STATUS, RP0			; Bank 0

;	clrf	UserBuf0		; lieber in EEPROM speichern, da Player doch oft abgeklemmt wird  - 030422
;	clrf	UserBuf1
;	clrf	UserBuf2
;	clrf	UserBuf3

	clrf	TXDiskNum
	clrf	TXTrackNum
	clrf	TXTrackRel
	clrf	TXPlayMode

	clrf	RXPointer

	bsf	STATUS, RP0
	movlw	TrackName0_def
	movwf	TrackName0
	movlw	TrackName1_def
	movwf	TrackName1
	movlw	TrackName2_def
	movwf	TrackName2
	movlw	TrackName3_def
	movwf	TrackName3
	movlw	TrackName4_def
	movwf	TrackName4
	movlw	TrackName5_def
	movwf	TrackName5
	movlw	TrackName6_def
	movwf	TrackName6
	movlw	TrackName7_def
	movwf	TrackName7

	movlw	TrackName8_def
	movwf	TrackName8
	movlw	TrackName9_def
	movwf	TrackName9
	movlw	TrackNameA_def
	movwf	TrackNameA
	movlw	TrackNameB_def
	movwf	TrackNameB
	movlw	TrackNameC_def
	movwf	TrackNameC
	movlw	TrackNameD_def
	movwf	TrackNameD
	movlw	TrackNameE_def
	movwf	TrackNameE
	movlw	TrackNameF_def
	movwf	TrackNameF

	movlw	DiskName0_def
	movwf	DiskName0
	movlw	DiskName1_def
	movwf	DiskName1
	movlw	DiskName2_def
	movwf	DiskName2
	movlw	DiskName3_def
	movwf	DiskName3
	movlw	DiskName4_def
	movwf	DiskName4
	movlw	DiskName5_def
	movwf	DiskName5
	movlw	DiskName6_def
	movwf	DiskName6
	movlw	DiskName7_def
	movwf	DiskName7

	movlw	DiskName8_def
	movwf	DiskName8
	movlw	DiskName9_def
	movwf	DiskName9
	movlw	DiskNameA_def
	movwf	DiskNameA
	movlw	DiskNameB_def
	movwf	DiskNameB
	movlw	DiskNameC_def
	movwf	DiskNameC
	movlw	DiskNameD_def
	movwf	DiskNameD
	movlw	DiskNameE_def
	movwf	DiskNameE
	movlw	DiskNameF_def
	movwf	DiskNameF
	bcf	STATUS, RP0

	IFDEF	PIC_16F62x
	movlw	0x07				; Disable the Comparitors
	movwf	CMCON
	ENDIF

	clrf	PORTA				; Configure Ports
	bsf	PORTA, 1			; Switch off attached player
	clrf	PORTB				; Clear PORTB
	bsf	STATUS, RP0
	movlw	0; 0xEF
	movwf	TRISA				; Port A
	movlw	0x6e; 0x03
	movwf	TRISB				; Port B - even if not used

	clrf	PIE1				; disable all interrupts
	bcf	STATUS, RP0

	IFDEF	SERIAL				; Code to enable serial port
	bsf	STATUS, RP0			; Select BANK1
	movlw	baud_val			; Set baud rate
	movwf	SPBRG

	movlw	0x04	;0x26			; Set transmit options:
	movwf	TXSTA				; !TX9 & !TXEN & ASYNC & BRGH
	;set TXIE if interrupt desired
	bsf	TXSTA, TXEN			; enable transmitter (sets TXIF=1!)

	bcf	STATUS, RP0			; Select register bank 0
	clrf	PIR1				; clear all interrupt flags

	movlw	0xC0				; Set receive options (SPEN & RX9)
	movwf	RCSTA

	bsf	STATUS, RP0			; Select BANK1
	bsf	PIE1, RCIE			; enable RX interrupt
	bsf	PIE1, EEIE			; enable EEPROM interrupt
	bcf	STATUS, RP0			; Select BANK0

	bsf	RCSTA, CREN			; enable continuous receive

	movlw	0xC0				; Enable global interrupt & 
	movwf	INTCON				; peripheral interrupt
	ENDIF

BusReset:
	btfsc	PORTA, 1			; Is player running?
	goto	BusResetSkipEE
	bsf	PORTA, 1			; Switch off attached player

	bsf	STATUS, RP0			; Store UserBuf0 to EEPROM
	clrf	EEADR
	bcf	STATUS, RP0
	movf	UserBuf0, W
	call	EEWrite
	bsf	INTCON, GIE			; EEWrite disables interrupts

BusResetSkipEE:
	clrf	MyId 				; Set up initial values
	clrf	MyBit
	movlw	Display0			; Start of display buffer
	movwf	DisplayW
	movwf	DisplayR
	movlw	0x80				; Idle
	movwf	State

	goto	ReceiveBreak
